# -*-python-*-

__package__    = "pocketindex/index.py"
__version__    = "0.1"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/pocketindex"
__date__       = "$Date: 2008/11/21 17:55:18 $"
__copyright__  = "Copyright (c) 2007-2008 Aaron Straup Cope. BSD license : http://www.modestmaps.com/license.txt"

import cairo
import Image
import math

class pocketindex :

    def __init__ (self, width, height, margin=30) :
        self.page_width = width
        self.page_height = height
        self.margin = margin
        self.pages = []
        self.legend = None
        
    #
    #
    #
    
    def create (self, data, legend=None) :

        if legend :
            self.legend = legend

        #
        
        (surface, ctx, x, y) = self.start_page()
    
        # why one-liners are kind of dumb; ensure
        # a constant line height...
        
        ln_height = int(max(map(lambda m: m[1], map(lambda l: self.measure_text(ctx, l['label']), data))) * 1.5)
        
        data.sort(lambda m, n: cmp(m['label'].lower(), n['label'].lower()))
        sz_dot = self.measure_text(ctx, ".")

        #

        tagged = []
        untagged = []
        
        tags_idx = {}

        for stuff in data:

            if not stuff.has_key('tags') :
                untagged.append(stuff)
                continue

            if len(stuff['tags']) == 0 :
                untagged.append(stuff)
                continue

            for t in stuff['tags'] :

                if not tags_idx.has_key(t):
                    tags_idx[t] = []
                    
                tags_idx[t].append(stuff)

        #

        tags_keys = tags_idx.keys()
        tags_keys.sort()

        for t in tags_keys :
            tagged.append({'header':t})

            for stuff in tags_idx[t] :
                tagged.append(stuff)

        if len(tagged) :
            tagged.append({'header':'unfiled'})

            for stuff in untagged :
                tagged.append(stuff)

            data = tagged
        else :
            data = untagged
            
        # 

        for stuff in data:

            if stuff.has_key('header') :

                header = stuff['header'].upper()
                sz_header = self.measure_text(ctx, header)

                y += int(ln_height / 2)
                
                ctx.move_to(x, y)
                ctx.show_text(header)

                y += int(ln_height / 2)

            else :
                label = stuff['label']
                value = stuff['value']
            
                sz_label = self.measure_text(ctx, label)
                sz_value = self.measure_text(ctx, value)
                
                stop = self.page_width - (sz_value[0] + self.margin)    
                
                while sz_label[0] > (stop - sz_value[0]) :
                    label = label[:-1]
                    sz_label = self.measure_text(ctx, label)

                ctx.move_to(x, y)
                ctx.show_text(unicode(label))

                x = x + (sz_label[0] + (sz_dot[0] * 2))

                while x < stop :
                    ctx.move_to(x, y)
                    ctx.show_text(".")
                    x = x + (sz_dot[0] * 2)
                    
                ctx.move_to(x, y)
                ctx.show_text(unicode(value))

            x = self.margin
            y = y + ln_height

            if y > (self.page_height - (self.margin * 2)) :
                self.write_page(surface)
                (surface, ctx, x, y) = self.start_page()    

        self.write_page(surface)
        return self.pages
        
    #
    #
    #
    
    def measure_text (self, ctx, txt):
        
        xbearing, ybearing, width, height, xadvance, yadvance = (
            ctx.text_extents(unicode(txt)))
        
        return (width, height)

    #
    #
    #

    def start_page (self) :

        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, self.page_width, self.page_height)
        ctx = cairo.Context(surface)

        # the pycairo docs are kind of useless sometimes
        # http://www.gnome.org/~federico/docs/file-chooser-profile/code/plot-languages.py
        
        font_face = "Helvetica Bold"
        font_size = 11

        ctx.select_font_face(font_face, cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
        ctx.set_font_size(font_size)

        ctx.move_to(0, 0)
        ctx.rectangle(0, 0, self.page_width, self.page_height)
        ctx.set_source_rgb(255, 255, 255)
        ctx.fill()

        ctx.set_source_rgb(0, 0, 0)

        x = self.margin
        y = int(self.margin * 1.5)

        #
        
        if self.legend :

            if self.legend.has_key('top_right') :

                text = self.legend['top_right']
                sz_text = self.measure_text(ctx, text)
                
                x = self.page_width - int(sz_text[0] + self.margin)
                
                ctx.move_to(x, y)
                ctx.show_text(text)
                
                x = self.margin
                y = y + int(sz_text[1] * 1.5)
            
        #
        
        ctx.move_to(x, y)

        return (surface, ctx, x, y)

    #
    #
    #
    
    def write_page (self, surface) :
        png = self.cairo2pil(surface)
        self.pages.append(png)

    #
    #
    #
    
    def cairo2pil(self, surface) :
        mode='RGBA'
        width = surface.get_width()
        height = surface.get_height()
        return Image.frombuffer(mode, (width, height), surface.get_data(), "raw", mode, 0, 1)

    #
    #
    #
    
if __name__ == '__main__' :

    width = 300
    height = 500
    
    legend = {'top_right':'test 2'}
    
    data = [
        {'label': 'test1', 'value': 'O HAI', 'tags':['lol', 'cat']},
        {'label': 'test2', 'value': 'O RLY',}
        ]

    idx = pocketindex(width, height)
    pages = idx.create(data, legend)

    # pages[0].save("/home/asc/Desktop/test.png")
